/*****************************************************************************
 *  Rohde&Schwarz CMW500 Wideband Radio Communication Tester instrument driver
 *  Audio Measurement
 *  Original Release: November 2013
 *  By: Petr Zavodny
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include "rscmwam.h"
#include <math.h>

#include "time.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
    #define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
        #include <windows.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString AudioTypeArr[] = {"Analog","Digital",NULL};
static ViString AudioSignalTypeArr[] = {"ANAL","DIG", NULL};
static ViString FrequencyLowPassFilterArr[] = {"OFF","LP3","LP4","LP15", NULL};
static ViString FrequencyHighPassFilterArr[] = {"OFF","H6","H50","H300", NULL};
static ViString WeightFilterArr[] = {"OFF","AWE","CCIT","CMSG", NULL};
static ViString switchArr[] = {"OFF","ON",NULL};
static ViString resultTypeArr[] = {"CURR","AVER","MAX","MIN","SDEV","EXTR", NULL};
static ViString limitCheckArr[] = {"LOW","UPP", NULL};
static ViString logMessageCategoryArr[] = {"INFO","WARN","ERR","CONT", NULL};
static ViString recSourceArr[] = {"ANAL","DIG","SPE", NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rscmwam_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rscmwam_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rscmwam_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwam_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr *)rscmwam_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rscmwam", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        100000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSCMWAM_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 3.4.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 3, 4, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rscmwam_RsCoreRepCapTable));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "CMW100A", false : "CMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSCMWAM_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSCMWAM_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rscmwam_reset(instrSession));
    }
    else
    {
        checkErr(rscmwam_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSCMWAM_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties->autoSystErrQuery == VI_FALSE)
        {
            sessionProperties->autoSystErrQuery = VI_TRUE;
            (void)rscmwam_CheckStatus(instrSession);
            sessionProperties->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN Configures the signal routing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR scenario/Selects active routing scenario.
/// HIPAR master/Selects the master for non-standalone scenarios.
ViStatus _VI_FUNC rscmwam_ActivateAudioScenario (ViSession instrSession,
                                                 ViInt32 instance,
                                                 ViInt32 scenario,
                                                 ViString master)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    switch (scenario)
    {
        case RSCMWAM_VAL_SCENARIO_AUDIO_MEAS_GEN:
            viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_AND_GENERATOR, NULL),
            		3, "Audio Measurement and Generator");
        break;
        case RSCMWAM_VAL_SCENARIO_EXT_ANALOG_SPEECH:
            viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_EXTERNAL_ANALOG_SPEECH_ANALYSIS, master),
            		4, "Master");
            break;
        case RSCMWAM_VAL_SCENARIO_EXT_DIGITAL_SPEECH:
            viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_EXTERNAL_DIGITAL_SPEECH_ANALYSIS, master),
            		4, "Master");
            break;
        case RSCMWAM_VAL_SCENARIO_EXT_ANALOG_STEREO:
            viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_ACTIVATE_STEREO_EXTERNAL_ANALOG_SPEECH_ANALYSIS, master),
            		4, "Master");
            break;
        case RSCMWAM_VAL_SCENARIO_EXT_DIGITAL_STEREO:
            viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_ACTIVATE_STEREO_EXTERNAL_DIGITAL_SPEECH_ANALYSIS, master),
            		4, "Master");
            break;
        case RSCMWAM_VAL_SCENARIO_MIC_SPEAKER_TEST:
            viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_MICROPHONE_AND_SPEAKER_TEST, master),
            		4, "Master");
            break;
		case RSCMWAM_VAL_SCENARIO_WAVEFORM_TEST:
            viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_WAVEFORM_TEST, NULL),
            		3, "Audio Measurement and Generator");
        break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, scenario), 3, "Scenario");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the scenario Audio Measurement and Generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR master/Sets the master.
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementandGenerator(
    ViSession   instrSession,
    ViInt32 instance,
    ViString master
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    master;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_AND_GENERATOR, NULL),
    		3, "Master");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates the scenario External Analog Speech Analysis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_EXTERNAL_ANALOG_SPEECH_ANALYSIS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ROUTe:AUDio:SCENario:EASPeech
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR master/Sets the master.
ViStatus _VI_FUNC rscmwam_ConfigureExternalAnalogSpeechAnalysis(
    ViSession   instrSession,
    ViInt32 instance,
    ViString master
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_EXTERNAL_ANALOG_SPEECH_ANALYSIS, master),
    		3, "Master");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates the scenario External Digital Speech Analysis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_EXTERNAL_DIGITAL_SPEECH_ANALYSIS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ROUTe:AUDio:SCENario:EDSPeech
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwam_ConfigureExternalDigitalSpeechAnalysis(
    ViSession   instrSession,
    ViInt32 instance,
    ViString master
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_EXTERNAL_DIGITAL_SPEECH_ANALYSIS, master),
    		3, "Master");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates the scenario "Microphone- and Speaker Test" and selects the
/// HIFN  master application.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_MICROPHONE_AND_SPEAKER_TEST
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ROUTe:AUDio:SCENario:MSTest
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR masterApplication/Activates the scenario "Microphone- and Speaker Test" and selects the
/// HIPAR masterApplication/master application.
ViStatus _VI_FUNC rscmwam_ConfigureMicrophoneandSpeakerTest(
    ViSession   instrSession,
    ViInt32 instance,
    ViString    masterApplication
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_MICROPHONE_AND_SPEAKER_TEST, masterApplication),
    		3, "Master Application");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a list of all possible master strings. The used master
/// HIFN  application is selected via the parameter <Master> in the scenario
/// HIFN  activation commands.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_POSSIBLE_MASTER_STRINGS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ROUTe:AUDio:SCENario:CATalog:MASTer?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR masterStrings/Returns a list of all possible master strings. The used master
/// HIPAR masterStrings/application is selected via the parameter <Master> in the scenario
/// HIPAR masterStrings/activation commands.
ViStatus _VI_FUNC rscmwam_QueryPossibleMasterStrings(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 arraySize,
    ViChar _VI_FAR  masterStrings[]
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_GetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_POSSIBLE_MASTER_STRINGS,
    		arraySize, masterStrings),
    		3, "Master Strings");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adjusts the clock of the audio board.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_CLOCK_DRIFT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:CDRift
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR clockDrift/Adjusts the clock of the audio board.
ViStatus _VI_FUNC rscmwam_ConfigureClockDrift(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    clockDrift
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_CLOCK_DRIFT, clockDrift),
    		3, "Clock Drift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Audio Measurement Statistics Settings
 * Purpose:  Configure the statistical settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementStatisticsSettings (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 audioType,
                                                                       ViInt32 repetition,
                                                                       ViInt32 stopCondition)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_REPETITION_MODE, repetition),
    		4, "Repetition Mode");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_STOP_CONDITION, stopCondition),
    		5, "Stop Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Audio Measurement Statistics Counter
 * Purpose:  Specifies the statistic counts for all audio signal types.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementStatisticsCounter (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 audioType,
                                                                      ViInt32 singleTone,
                                                                      ViInt32 multitone,
                                                                      ViInt32 FFTNoise)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command [100] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, singleTone, 1, 1000),
    		4, "Single Tone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multitone, 1, 1000),
    		5, "Multitone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, FFTNoise, 1, 1000),
    		6, "FFT Noise");

    sprintf (command, "CONF:AUD:MEAS%ld:%s:SCO %ld,%ld,%ld", instance, AudioSignalTypeArr[audioType], singleTone, multitone, FFTNoise);

    checkErr(RsCore_Write(instrSession, command));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Query Audio Measurement Signal Type
 * Purpose:  Queries the currently used or expected signal type, resulting
 *           from the generator coupling.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryAudioMeasurementSignalType (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 audioType,
                                                           ViInt32 *signalType)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_GetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_SIGNAL_TYPE, signalType),
    		4, "Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Audio Measurement Analog Input Settings
 * Purpose:  Sets the input path.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementAnalogInputSettings (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 pathCoupling,
                                                                        ViInt32 autoRanging,
                                                                        ViReal64 maximumLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_ANALOG_PATH_COUPLING, pathCoupling),
    		3, "Path Coupling");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_ANALOG_AUTO_RANGING, autoRanging),
    		4, "Auto Ranging");

    if (autoRanging == VI_FALSE)
    {
    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_ANALOG_MAXIMUM_LEVEL, maximumLevel),
    		5, "Maximum Level");
    }
    else
    {
    }
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Audio Measurement Lowpass Filter
 * Purpose:  Configures the cutoff frequency of the lowpass filter for the
 *           individual audio signal types.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementLowpassFilter (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 audioType,
                                                                  ViInt32 frequencySingleTone,
                                                                  ViInt32 frequencyMultitone,
                                                                  ViInt32 frequencyFFTNoise)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command [100] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencySingleTone, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_CUTOFF_FREQUENCY_LP15),
    		4, "Frequency Single Tone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencyMultitone, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_CUTOFF_FREQUENCY_LP15),
    		5, "Frequency Multitone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencyFFTNoise, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_CUTOFF_FREQUENCY_LP15),
    		6, "Frequency FFT Noise");

    sprintf (command, "CONF:AUD:MEAS%ld:%s:FILT:LPAS %s, %s, %s", instance, AudioSignalTypeArr[audioType], FrequencyLowPassFilterArr[frequencySingleTone], FrequencyLowPassFilterArr[frequencyMultitone], FrequencyLowPassFilterArr[frequencyFFTNoise]);

    checkErr(RsCore_Write(instrSession, command));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Audio Measurement Highpass Filter
 * Purpose:  Configures the cutoff frequency of the highpass filter for the
 *           individual audio signal types.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementHighpassFilter (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViInt32 audioType,
                                                                   ViInt32 frequencySingleTone,
                                                                   ViInt32 frequencyMultitone,
                                                                   ViInt32 frequencyFFTNoise)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command [100] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencySingleTone, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_CUTOFF_FREQUENCY_H300),
    		4, "Frequency Single Tone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencyMultitone, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_CUTOFF_FREQUENCY_H300),
    		5, "Frequency Multitone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencyFFTNoise, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_CUTOFF_FREQUENCY_H300),
    		6, "Frequency FFT Noise");

    sprintf (command, "CONF:AUD:MEAS%ld:%s:FILT:HPAS %s, %s, %s", instance, AudioSignalTypeArr[audioType], FrequencyHighPassFilterArr[frequencySingleTone], FrequencyHighPassFilterArr[frequencyMultitone], FrequencyHighPassFilterArr[frequencyFFTNoise]);

    checkErr(RsCore_Write(instrSession, command));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Audio Measurement Weighting Filter
 * Purpose:  Configures the weighting filter for the individual audio signal
 *           types.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementWeightingFilter (ViSession instrSession,
                                                                    ViInt32 instance,
                                                                    ViInt32 audioType,
                                                                    ViInt32 singleTone,
                                                                    ViInt32 multitone,
                                                                    ViInt32 FFTNoise)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command [100] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, singleTone, RSCMWAM_VAL_WEIGHTING_FILTER_OFF, RSCMWAM_VAL_WEIGHTING_FILTER_CMSG),
    		4, "Single Tone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multitone, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_WEIGHTING_FILTER_CMSG),
    		5, "Multitone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, FFTNoise, RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF, RSCMWAM_VAL_WEIGHTING_FILTER_CMSG),
    		6, "FFT Noise");

    sprintf (command, "CONF:AUD:MEAS%ld:%s:FILT:WEIG %s, %s, %s", instance, AudioSignalTypeArr[audioType], WeightFilterArr[singleTone], WeightFilterArr[multitone], WeightFilterArr[FFTNoise]);

    checkErr(RsCore_Write(instrSession, command));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Audio Measurement Init
 * Purpose:  Starts or restarts the measurement; the R&S CMW 500 enters the
 *           "RUN" state.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_AudioMeasurementInit (ViSession instrSession,
                                                ViInt32 instance,
                                                ViInt32 audioType)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_INIT, NULL),
    		3, "Measurement Init");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Audio Measurement Abort
 * Purpose:  Causes a running measurement to stop immediately; the R&S CMW 500
 *           enters the "OFF" state.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_AudioMeasurementAbort (ViSession instrSession,
                                                 ViInt32 instance,
                                                 ViInt32 audioType)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_ABORT, NULL),
    		3, "Measurement Abort");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Audio Measurement Stop
 * Purpose:  Causes a running measurement to stop after the current
 *           evaluation period is terminated and valid results are available;
 *           the R&S CMW 500 enters the "RDY" state.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_AudioMeasurementStop (ViSession instrSession,
                                                ViInt32 instance,
                                                ViInt32 audioType)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_AUDIO_MEASUREMENT_STOP, NULL),
    		3, "Measurement Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Query Audio Measurement Status
 * Purpose:  Queries the main measurement state and the measurement
 *           substates.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryAudioMeasurementStatus (ViSession instrSession,
                                                       ViInt32 instance,
                                                       ViInt32 audioType,
                                                       ViInt32 *mainState,
                                                       ViInt32 *syncState,
                                                       ViInt32 *resourcesState)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");

    sprintf (command, "FETC:AUD:MEAS%ld:%s:STAT:ALL?", instance, AudioSignalTypeArr[audioType]);

    checkErr(rscmwam_QueryMeasStatus(instrSession, command, mainState, syncState,
        resourcesState));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Single Tone Measurement Statistics Count
 * Purpose:  Specifies the statistic count for single tone measurements. The
 *           statistic count is equal to the number of measurement intervals
 *           per single shot.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementStatisticsCount (ViSession instrSession,
                                                                         ViInt32 instance,
                                                                         ViInt32 audioType,
                                                                         ViInt32 statisticCount)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_SINGLE_TONE_STATISTIC_COUNT, statisticCount),
    		4, "Statistic Count");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Single Tone Measurement Filter Settings
 * Purpose:  Sets The single tone measurement filter settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementFilterSettings (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 audioType,
                                                                        ViReal64 distorsionFrequency,
                                                                        ViInt32 lowPass,
                                                                        ViInt32 highPass,
                                                                        ViInt32 weighting)
{
    ViStatus error = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_SINGLE_TONE_FREQUENCY_DISTORTION, distorsionFrequency),
    		4, "Distorsion Frequency");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_SINGLE_TONE_LOW_PASS_FILTER, lowPass),
    		5, "Low Pass");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_SINGLE_TONE_HIGH_PASS_FILTER, highPass),
    		6, "High Pass");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_SINGLE_TONE_WEIGHTING_FILTER, weighting),
    		7, "Weighting");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a window function for the single tone spectrum measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR audioType/Selects the audio signal type.
/// HIPAR windowFunction/Configures a window function for the single tone spectrum measurement.
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementFilterWindowFunction(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 audioType,
    ViInt32 windowFunction
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_SINGLE_TONE_WINDOW_FUNCTION, windowFunction),
    		4, "Window Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Single Tone Measurement Filter BandPass
 * Purpose:  Sets the center frequency and bandwidth of the bandpass filter
 *           for single tone measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementFilterBandPass (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 audioType,
                                                                        ViReal64 centerFrequency,
                                                                        ViReal64 bandwidth)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command [100] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, centerFrequency, 0.0, 24.0E+3), 4, "Center Frequency");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, bandwidth, 0.0, 24.0E+3), 5, "Bandwidth");

    sprintf (command, "CONF:AUD:MEAS%ld:%s:SING:FILT:BAND %.12lG, %.12lG", instance, AudioSignalTypeArr[audioType], centerFrequency, bandwidth);

    checkErr(RsCore_Write(instrSession, command));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Single Tone Analog Measurement Limits
 * Purpose:  Configures the limits for analog single tone measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneAnalogMeasurementLimits (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViReal64 weightRMSUpper,
                                                                      ViBoolean weightRMSEnabled,
                                                                      ViReal64 bandRMSUpper,
                                                                      ViBoolean bandRMSEnabled,
                                                                      ViReal64 THDPercentUpper,
                                                                      ViBoolean THDPercentEnabled,
                                                                      ViReal64 THDNUpper,
                                                                      ViBoolean THDNEnabled,
                                                                      ViReal64 SNRLow,
                                                                      ViBoolean SNREnabled)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command [RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, weightRMSUpper, 0.0, 5.0), 3, "Weight RMS Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, bandRMSUpper, 0.0, 5.0), 5, "Band RMS Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, THDPercentUpper, 0.5, 100.0), 7, "THD Percent Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, THDNUpper, 0.5, 100.0), 9, "THD+N Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, SNRLow, 0.0, 100.0), 11, "SNR Low");

    sprintf (command, "CONF:AUD:MEAS%ld:ANAL:SING:LIM %7.6lf,%s,%7.6lf,%s,%7.6lf,%s,%7.6lf,%s,%7.6lf,%s", instance, weightRMSUpper, switchArr[weightRMSEnabled],
        bandRMSUpper, switchArr[bandRMSEnabled], THDPercentUpper, switchArr[THDPercentEnabled], THDNUpper, switchArr[THDNEnabled], SNRLow, switchArr[SNREnabled]);

    checkErr(RsCore_Write(instrSession, command));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Single Tone Digital Measurement Limits
 * Purpose:  Configures the limits for digital single tone measurements
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneDigitalMeasurementLimits (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViReal64 weightRMSUpper,
                                                                       ViBoolean weightRMSEnabled,
                                                                       ViReal64 bandRMSUpper,
                                                                       ViBoolean bandRMSEnabled,
                                                                       ViReal64 THDPercentUpper,
                                                                       ViBoolean THDPercentEnabled,
                                                                       ViReal64 THDNUpper,
                                                                       ViBoolean THDNEnabled,
                                                                       ViReal64 SNRLow,
                                                                       ViBoolean SNREnabled)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command [RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, weightRMSUpper, 0.0, 1.0), 3, "Weight RMS Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, bandRMSUpper, 0.0, 1.0), 5, "Band RMS Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, THDPercentUpper, 0.5, 100.0), 7, "THD Percent Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, THDNUpper, 0.5, 100.0), 9, "THD+N Upper");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, SNRLow, 0.0, 100.0), 11, "SNR Low");

    sprintf (command, "CONF:AUD:MEAS%ld:DIG:SING:LIM %.6le,%s,%.6le,%s,%.6le,%s,%.6le,%s,%.6le,%s", instance, weightRMSUpper, switchArr[weightRMSEnabled],
        bandRMSUpper, switchArr[bandRMSEnabled], THDPercentUpper, switchArr[THDPercentEnabled], THDNUpper, switchArr[THDNEnabled], SNRLow, switchArr[SNREnabled]);

    checkErr(RsCore_Write(instrSession, command));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  Enables or disables the evaluation of the single tone spectrum results
/// HIFN  and shows or hides the spectrum diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR audioType/Selects the audio signal type.
/// HIPAR spectrumState/Enables or disables the evaluation of the single tone spectrum results
/// HIPAR spectrumState/and shows or hides the spectrum diagram.
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneSpectrumState(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 audioType,
    ViBoolean   spectrumState
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViBoolean(instrSession, repCap, RSCMWAM_ATTR_SINGLE_TONE_SPECTRUM_STATE, spectrumState),
    		4, "Spectrum State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read Single Tone Analog Measurement Limits
 * Purpose:  Returns the analog single tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadSingleToneAnalogMeasurementLimits (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 resultType,
                                                                 ViInt32 arraySize,
                                                                 ViInt32 *reliability,
                                                                 ViReal64 results[],
                                                                 ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_EXTREME),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "READ:AUD:MEAS%ld:ANAL:SING:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Fetch Single Tone Analog Measurement Limits
 * Purpose:  Returns the analog single tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchSingleToneAnalogMeasurementLimits (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 resultType,
                                                                  ViInt32 arraySize,
                                                                  ViInt32 *reliability,
                                                                  ViReal64 results[],
                                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_EXTREME),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "FETC:AUD:MEAS%ld:ANAL:SING:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read Single Tone Digital Measurement Limits
 * Purpose:  Returns the digital single tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadSingleToneDigitalMeasurementLimits (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 resultType,
                                                                  ViInt32 arraySize,
                                                                  ViInt32 *reliability,
                                                                  ViReal64 results[],
                                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_EXTREME),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "READ:AUD:MEAS%ld:DIG:SING:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Fetch Single Tone Digital Measurement Limits
 * Purpose:  Returns the digital single tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchSingleToneDigitalMeasurementLimits (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViInt32 resultType,
                                                                   ViInt32 arraySize,
                                                                   ViInt32 *reliability,
                                                                   ViReal64 results[],
                                                                   ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_EXTREME),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "FETC:AUD:MEAS%ld:DIG:SING:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Multitone Measurement Statistics Count
 * Purpose:  Sets the statistic count for multitone measurements. The
 *           statistic count is equal to the number of measurement intervals
 *           per single shot.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneMeasurementStatisticsCount (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 audioType,
                                                                        ViInt32 statisticCount)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_MULTITONE_STATISTIC_COUNT, statisticCount),
    		4, "Statistic Count");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Multitone Measurement Result Mode
 * Purpose:  Selects whether the multitone results are provided as absolute
 *           levels (dBV or dBFS) or as relative levels (dB).
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneMeasurementResultMode (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViInt32 audioType,
                                                                   ViInt32 resultMode)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_MULTITONE_RESULT_MODE, resultMode),
    		4, "Result Mode");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Multitone Measurement Filter Settings
 * Purpose:  Configures The multitone measurement filter settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneMeasurementFilterSettings (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 audioType,
                                                                       ViInt32 lowPass,
                                                                       ViInt32 highPass,
                                                                       ViInt32 weighting)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_MULTITONE_LOW_PASS_FILTER, lowPass),
    		4, "Low Pass");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_MULTITONE_HIGH_PASS_FILTER, highPass),
    		5, "High Pass");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_MULTITONE_WEIGHTING_FILTER, weighting),
    		6, "Weighting");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Multitone Analog Measurement Limits List
 * Purpose:  Configures the limits list for analog multitone measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneAnalogMeasurementLimitsList (ViSession instrSession,
                                                                         ViInt32 instance,
                                                                         ViInt32 limitType,
                                                                         ViInt32 arraySize,
                                                                         ViReal64 limits[],
                                                                         ViBoolean states[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = NULL;
    ViInt32     i = 0;
    ViChar*     p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, RSCMWAM_VAL_LIMIT_LOWER, RSCMWAM_VAL_LIMIT_UPPER),
    		3, "Limit Type");

    buffer = (ViChar*) malloc (100 + arraySize * 50);
    p2buf = buffer + sprintf (buffer, "CONF:AUD:MEAS%ld:ANAL:MULT:LIM:%s ", instance, limitCheckArr[limitType]);

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%.12lG,%s,", limits[i], states[i] ? "ON" : "OFF");

    *--p2buf = '\0';

    checkErr(RsCore_Write(instrSession, buffer));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Multitone Analog Measurement Limits
 * Purpose:  Configures an upper and lower limits for a selected tone of the
 *           multitone list.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneAnalogMeasurementLimits (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 tone,
                                                                     ViInt32 limitType,
                                                                     ViReal64 limit,
                                                                     ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tone, 1, 20),
    		3, "Tone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, RSCMWAM_VAL_LIMIT_LOWER, RSCMWAM_VAL_LIMIT_UPPER),
    		4, "Limit Type");

    if (limitType == RSCMWAM_VAL_LIMIT_LOWER)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:MEAS%ld:ANAL:MULT:TONE%ld:LIM:LOW %.12lG, %s", instance, tone, limit, switchArr[state]);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:MEAS%ld:ANAL:MULT:TONE%ld:LIM:UPP %.12lG, %s", instance, tone, limit, switchArr[state]);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Multitone Digital Measurement Limits List
 * Purpose:  Configures the limits list for the digital multitone
 *           measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneDigitalMeasurementLimitsList (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 limitType,
                                                                          ViInt32 arraySize,
                                                                          ViReal64 limits[],
                                                                          ViBoolean states[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = NULL;
    ViInt32     i = 0;
    ViChar*     p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, RSCMWAM_VAL_LIMIT_LOWER, RSCMWAM_VAL_LIMIT_UPPER),
    		3, "Limit Type");

    buffer = (ViChar*) malloc (100 + arraySize * 50);
     p2buf = buffer + sprintf (buffer, "CONF:AUD:MEAS%ld:DIG:MULT:LIM:%s ", instance, limitCheckArr[limitType]);

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%.12lG,%s,", limits[i], states[i] ? "ON" : "OFF");

    *--p2buf = '\0';

    checkErr(RsCore_Write(instrSession, buffer));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Multitone Digital Measurement Limits
 * Purpose:  Configures an upper and lower limits for a selected tone of the
 *           multitone list.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneDigitalMeasurementLimits (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 tone,
                                                                      ViInt32 limitType,
                                                                      ViReal64 limit,
                                                                      ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tone, 1, 20),
    		3, "Tone");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, RSCMWAM_VAL_LIMIT_LOWER, RSCMWAM_VAL_LIMIT_UPPER),
    		4, "Limit Type");

    if (limitType == RSCMWAM_VAL_LIMIT_LOWER)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:MEAS%ld:DIG:MULT:TONE%ld:LIM:LOW %.12lG, %s", instance, tone, limit, switchArr[state]);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:MEAS%ld:DIG:MULT:TONE%ld:LIM:UPP %.12lG, %s", instance, tone, limit, switchArr[state]);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read Multitone Analog Measurement Limits
 * Purpose:  Returns the analog multitone tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadMultitoneAnalogMeasurementLimits (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 resultType,
                                                                ViInt32 arraySize,
                                                                ViInt32 *reliability,
                                                                ViReal64 results[],
                                                                ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_SDEV),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "READ:AUD:MEAS%ld:ANAL:MULT:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multitone Analog Measurement Limits
 * Purpose:  Returns the analog multitone tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchMultitoneAnalogMeasurementLimits (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 resultType,
                                                                 ViInt32 arraySize,
                                                                 ViInt32 *reliability,
                                                                 ViReal64 results[],
                                                                 ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_SDEV),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "FETC:AUD:MEAS%ld:ANAL:MULT:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read Multitone Digital Measurement Limits
 * Purpose:  Returns the digital multitone tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadMultitoneDigitalMeasurementLimits (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 resultType,
                                                                 ViInt32 arraySize,
                                                                 ViInt32 *reliability,
                                                                 ViReal64 results[],
                                                                 ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_SDEV),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "READ:AUD:MEAS%ld:DIG:MULT:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multitone Digital Measurement Limits
 * Purpose:  Returns the digital multitone tone measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchMultitoneDigitalMeasurementLimits (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 resultType,
                                                                  ViInt32 arraySize,
                                                                  ViInt32 *reliability,
                                                                  ViReal64 results[],
                                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_SDEV),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    sprintf (command, "FETC:AUD:MEAS%ld:DIG:MULT:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure FFT Noise Measurement Statistics Count
 * Purpose:  Specifies the statistic count for FFT noise measurements. The
 *           statistic count is equal to the number of measurement intervals
 *           per single shot.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureFFTNoiseMeasurementStatisticsCount (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 audioType,
                                                                       ViInt32 statisticCount)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_FFT_NOISE_STATISTIC_COUNT, statisticCount),
    		4, "Statistic Count");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure FFT Noise Measurement Filter Settings
 * Purpose:  Configures The FFT noise measurement filter settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureFFTNoiseMeasurementFilterSettings (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 audioType,
                                                                      ViInt32 lowPass,
                                                                      ViInt32 highPass,
                                                                      ViInt32 weighting)
{
    ViStatus error = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_FFT_NOISE_LOW_PASS_FILTER, lowPass),
    		4, "Low Pass");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_FFT_NOISE_HIGH_PASS_FILTER, highPass),
    		5, "High Pass");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_FFT_NOISE_WEIGHTING_FILTER, weighting),
    		6, "Weighting");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read FFT Noise Analog Measurement Level Results
 * Purpose:  Returns the analog fft noise measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseAnalogMeasurementLevelResults (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 resultType,
                                                                     ViInt32 arraySize,
                                                                     ViInt32 *reliability,
                                                                     ViReal64 level[],
                                                                     ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_MIN),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, level), 6, "Level");

    sprintf (command, "READ:AUD:MEAS%ld:ANAL:FFTN:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, level, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Fetch FFT Noise Analog Measurement Level Results
 * Purpose:  Returns the analog fft noise measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseAnalogMeasurementLevelResults (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 resultType,
                                                                      ViInt32 arraySize,
                                                                      ViInt32 *reliability,
                                                                      ViReal64 level[],
                                                                      ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_MIN),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, level), 6, "Level");

    sprintf (command, "FETC:AUD:MEAS%ld:ANAL:FFTN:%s?", instance,  resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, level, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read FFT Noise Analog Measurement Frequency Results
 * Purpose:  Returns the X-values of the traces measured with an analog FFT
 *           noise measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseAnalogMeasurementFrequencyResults (ViSession instrSession,
                                                                         ViInt32 instance,
                                                                         ViInt32 arraySize,
                                                                         ViInt32 *reliability,
                                                                         ViReal64 frequency[],
                                                                         ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 5, "Frequency");

    sprintf (command, "READ:AUD:MEAS%ld:ANAL:FFTN:FREQ?", instance);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, frequency, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read FFT Noise Analog Measurement Frequency Results
 * Purpose:  Returns the X-values of the traces measured with an analog FFT
 *           noise measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseAnalogMeasurementFrequencyResults (ViSession instrSession ,
                                                                          ViInt32 instance,
                                                                          ViInt32 arraySize,
                                                                          ViInt32 *reliability,
                                                                          ViReal64 frequency[],
                                                                          ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 5, "Frequency");

    sprintf (command, "FETC:AUD:MEAS%ld:ANAL:FFTN:FREQ?", instance);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, frequency, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read FFT Noise Analog Measurement Frequency Results
 * Purpose:  Returns the digital fft noise measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseDigitalMeasurementLevelResults (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 resultType,
                                                                      ViInt32 arraySize,
                                                                      ViInt32 *reliability,
                                                                      ViReal64 level[],
                                                                      ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_MIN),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, level), 6, "Level");

    sprintf (command, "READ:AUD:MEAS%ld:DIG:FFTN:%s?", instance, resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, level, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Fetch FFT Noise Digital Measurement Level Results
 * Purpose:  Returns the digital fft noise measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseDigitalMeasurementLevelResults (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 resultType,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 *reliability,
                                                                       ViReal64 level[],
                                                                       ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWAM_VAL_RESULT_CURRENT, RSCMWAM_VAL_RESULT_MIN),
    		3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, level), 6, "Level");

    sprintf (command, "FETC:AUD:MEAS%ld:DIG:FFTN:%s?", instance, resultTypeArr[resultType]);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, level, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read FFT Noise Digital Measurement Frequency Results
 * Purpose:  Returns the X-values of the traces measured with an digital fft
 *           noise measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseDigitalMeasurementFrequencyResults (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 arraySize,
                                                                          ViInt32 *reliability,
                                                                          ViReal64 frequency[],
                                                                          ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 6, "Frequency");

    sprintf (command, "READ:AUD:MEAS%ld:DIG:FFTN:FREQ?", instance);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, frequency, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read FFT Noise Digital Measurement Frequency Results
 * Purpose:  Returns the X-values of the traces measured with an digital fft
 *           noise measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseDigitalMeasurementFrequencyResults (ViSession instrSession,
                                                                           ViInt32 instance,
                                                                           ViInt32 arraySize,
                                                                           ViInt32 *reliability,
                                                                           ViReal64 frequency[],
                                                                           ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 6, "Frequency");

    sprintf (command, "FETC:AUD:MEAS%ld:DIG:FFTN:FREQ?", instance);
    checkErr(rscmwam_GetMeasTrace(instrSession, command, 15000, arraySize, reliability, frequency, returnedValues));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Generator Signal Type
 * Purpose:  Selects the type of the generated audio signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorSignalType (ViSession instrSession,
                                                        ViInt32 instance,
                                                        ViInt32 audioType,
                                                        ViInt32 signalType)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_SIGNAL_TYPE, signalType),
    		4, "Signal Type");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Generator Analyzer Coupling
 * Purpose:  Selects the audio measurement to which the audio generator is
 *           coupled.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorAnalyzerCoupling (ViSession instrSession,
                                                              ViInt32 instance,
                                                              ViInt32 audioType,
                                                              ViInt32 analyzerCoupling)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_ANALYZER_COUPLING, analyzerCoupling),
    		4, "Analyzer Coupling");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Generator Single Tone List
 * Purpose:  Specifies the frequency and level of the single tone to be
 *           generated.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorSingleToneList (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 audioType,
                                                            ViReal64 frequency,
                                                            ViReal64 level)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_SINGLE_TONE_FREQUENCY, frequency),
    		4, "Frequency");

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_SINGLE_TONE_LEVEL, level),
    		5, "Level");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Generator Multitone Crest Factor
 * Purpose:  Configures the crest factor of the generated multitone signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneCrestFactor (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 audioType,
                                                                  ViInt32 crestFactor)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_MULTITONE_CREST_FACTOR, crestFactor),
    		4, "Crest Factor");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Generator Multitone Level Selection
 * Purpose:  Selects a method for multitone level configuration.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneLevelSelection (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 audioType,
                                                                     ViInt32 levelSelection)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_MULTITONE_LEVEL_SELECTION, levelSelection),
    		4, "Level Selection");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Generator Multitone Total Level
 * Purpose:  Configures the total level of an analog and digital multitone
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneTotalLevel (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 audioType,
                                                                 ViReal64 totalLevel)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_MULTITONE_TOTAL_LEVEL, totalLevel),
    		4, "Total Level");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query Generator Multitone Total Level
 * Purpose:  Queries the total level of an analog and digital multitone
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryGeneratorMultitoneTotalLevel (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 audioType,
                                                             ViReal64 *totalLevel)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_GetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_QUERY_GENERATOR_MULTITONE_TOTAL_LEVEL, totalLevel),
    		4, "Total Level");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Generator Multitone Tone Settings
 * Purpose:  Sets the first and the last tone of the multitone to be used.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneToneSettings (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViInt32 audioType,
                                                                   ViInt32 startTone,
                                                                   ViInt32 stopTone)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_MULTITONE_TONE_START, startTone),
    		4, "Start Tone");

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_MULTITONE_TONE_STOP, stopTone),
    		5, "Stop Tone");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Generator Multitone Defined Tone List
 * Purpose:  Configures a selected tone of the multitone list of the analog
 *           and the digital generator.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneDefinedToneList (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 audioType,
                                                                      ViInt32 tone,
                                                                      ViReal64 frequency,
                                                                      ViReal64 level)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tone, 1, 20),
    		4, "Tone");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, frequency, 0.0, 21.0E+03), 5, "Frequency");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, level, 0.0, 5.0), 6, "Level");

    sprintf (command, "SOUR:AUD:GEN%ld:%s:MULT:TONE%ld:TDEF %.12lG, %.12lG", instance, AudioSignalTypeArr[audioType], tone, frequency, level);

    checkErr(RsCore_Write(instrSession, command));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Generator FFT Noise Level
 * Purpose:  Configures the frequency and level of the first n tones of the
 *           multitone list of the digital and the analog generator (n = 1 to
 *           20), the parameters are sent by pair {<Frequency>, <Level>}1,
 *           ..., {<Frequency>, <Level>}n
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneListAll (ViSession instrSession,
                                                              ViInt32 instance,
                                                              ViInt32 audioType,
                                                              ViInt32 arraySize,
                                                              ViReal64 frequency[],
                                                              ViReal64 level[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = NULL;
    ViInt32     i = 0;
    ViChar*     p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, audioType, RSCMWAM_VAL_ANALOG, RSCMWAM_VAL_DIGITAL),
    		3, "Audio Type");

    buffer = (ViChar*) malloc (100 + arraySize * 50);
     p2buf = buffer + sprintf (buffer, "SOUR:AUD:GEN%ld:%s:MULT:TDEF ", instance, AudioSignalTypeArr[audioType]);

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%.12lG,%.12lG,", frequency[i], level[i]);

    *--p2buf = '\0';

    checkErr(RsCore_Write(instrSession, buffer));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Generator Multitone List All
 * Purpose:  Configures the frequency and level of the first n tones of the
 *           multitone list of the digital and the analog generator (n = 1 to
 *           20), the parameters are sent by pair {<Frequency>, <Level>}1,
 *           ..., {<Frequency>, <Level>}n
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorFFTNoiseLevel (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 audioType,
                                                           ViReal64 level)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,%s", instance, AudioTypeArr[audioType]);

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_GENERATOR_FFT_NOISE_LEVEL, level),
    		4, "Level");

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the last event log entry.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgs_init or
/// HIPAR instrSession/rscmwgs_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the  Time
/// HIPAR bufferSize/Stamp and Event Message.
/// HIPAR timeStamp/Timestamp of the entry as string in the format "hh:mm:ss".
/// HIPAR eventCategory/Category of event log entry.
/// HIPAR message/Text string describing the event.
ViStatus _VI_FUNC rscmwam_QueryEventLogLastEntry(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 bufferSize,
    ViChar _VI_FAR  timeStamp[],
    ViInt32*    eventCategory,
    ViChar _VI_FAR  message[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViChar      *p2buf;
    ViChar      cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:AUD:SPE%ld:ELOG:LAST?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));
    checkErr(rscmwam_CheckStatus(instrSession));

    p2buf = strtok (pbuffer, ",\r\n");
    strncpy(timeStamp, p2buf, bufferSize);

    p2buf = strtok (NULL, ",\r\n");
    *eventCategory = RsCore_FindStringIndex(logMessageCategoryArr, p2buf);

    p2buf = strtok (NULL, ",\r\n");
    strncpy(message, p2buf, bufferSize);

Error:
    if(pbuffer != NULL) free(pbuffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the all event log entries.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgs_init or
/// HIPAR instrSession/rscmwgs_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Number of elements alcoated for array.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the  Time
/// HIPAR bufferSize/Stamp and Event Message.
/// HIPAR timeStamp/Comma separated list of entry timestamps in the format "hh:mm:ss".
/// HIPAR eventCategory/Array of event log category entries.
/// HIPAR message/Comma separated list describing each event.
/// HIPAR numberOfEntries/Number of returned entries.
ViStatus _VI_FUNC rscmwam_QueryEventLogAllEntries(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 arraySize,
    ViInt32 bufferSize,
    ViChar _VI_FAR  timeStamp[],
    ViInt32 eventCategory[],
    ViChar _VI_FAR  message[],
    ViInt32*    numberOfEntries
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
    ViInt32     TSBufferSize = bufferSize - 1;
    ViInt32     MsgBufferSize = bufferSize - 1;
    ViChar      cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:AUD:SPE%ld:ELOG:ALL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));
    checkErr(rscmwam_CheckStatus(instrSession));

    p2buf = strtok (pbuffer, ",\r\n");
    while(p2buf && (i < arraySize))
    {
        if(TSBufferSize)
        {
            strncat(timeStamp, p2buf, TSBufferSize);
            TSBufferSize -= (ViInt32) strlen(p2buf);

            strncat(timeStamp, ",", TSBufferSize--);
        }

        p2buf = strtok (NULL, ",\r\n");
        eventCategory[i] = RsCore_FindStringIndex(logMessageCategoryArr, p2buf);

        p2buf = strtok (NULL, ",\r\n");
        if(MsgBufferSize)
        {
            strncat(message, p2buf, MsgBufferSize);
            MsgBufferSize -= (ViInt32) strlen(p2buf);

            strncat(message, ",", MsgBufferSize--);
        }

        i++;
        p2buf = strtok (NULL, ",\r\n");
    }

    if(message[strlen(message)] == ',') message[strlen(message)] = '\0';
    *numberOfEntries = i;

Error:
    if(pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the A/D conversion for analog speech scenarios with an
/// HIFN  input/output signal at the AF connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR inputLevelFullScale/Analog input signal peak voltage corresponding to digital signal
/// HIPAR inputLevelFullScale/full-scale value.
/// HIPAR outputLevelFullScale/Analog output signal peak voltage corresponding to digital signal
/// HIPAR outputLevelFullScale/full-scale value.
ViStatus _VI_FUNC rscmwam_ConfigureSpeechAnalysisControl(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    inputLevelFullScale,
    ViReal64    outputLevelFullScale
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_SPEECH_ANALYSIS_INPUT_LEVEL, inputLevelFullScale),
    		3, "Input Level Full Scale");

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_SPEECH_ANALYSIS_OUTPUT_LEVEL, outputLevelFullScale),
    		4, "Output Level Full Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the cutoff frequency of the highpass filter, located between the AF IN connectors and the speech encoder.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwlm_init or
/// HIPAR instrSession/rscmwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR highPass/Configures the cutoff frequency of the highpass filter, located between the AF IN connectors and the speech encoder.
ViStatus _VI_FUNC rscmwam_ConfigureSpeechAnalysisFilter (ViSession instrSession,
                                                         ViInt32 instance,
                                                         ViInt32 highPass)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_SPEECH_ANALYSIS_FILTER_HIGH_PASS, highPass),
    		3, "High Pass");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the recording:
/// HIFN     INITiate... starts or continues the recording. The state switches to
/// HIFN  'RUN'.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_RECORDING_INITIATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     INITiate:AUDio:WAV:RECord
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwam_RecordingInit(
	ViSession	instrSession,
	ViInt32	instance
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_RECORDING_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Aborts the recording:
/// HIFN     ABORt... stops the recording and writes the waveform file. The state
/// HIFN  switches to 'OFF'. Allocated resources are released.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_RECORDING_ABORT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ABORt:AUDio:WAV:RECord
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwam_RecordingAbort(
	ViSession	instrSession,
	ViInt32	instance
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_RECORDING_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Stops the recording:
/// HIFN     STOP... pauses the recording. The state switches to 'RDY'. The
/// HIFN  resources remain allocated.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_RECORDING_STOP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     STOP:AUDio:WAV:RECord
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwam_RecordingStop(
	ViSession	instrSession,
	ViInt32	instance
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_RECORDING_STOP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the amount of already recorded data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_RECORDING_AMOUNT_OF_RECORDED_DATA
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:AUDio:WAV:RECord:PROGress:DWRitten?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR recordedData/Displays the amount of already recorded data.
ViStatus _VI_FUNC rscmwam_QueryRecordingAmountOfRecordedData(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64*	recordedData
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwam_GetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_RECORDING_AMOUNT_OF_RECORDED_DATA, recordedData));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the main measurement state and the measurement substates. Both
/// HIFN  measurement substates are relevant for running measurements only.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:AUDio:WAV:RECord:STATe:ALL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR mainState/Returns main state.
/// HIPAR syncState/Returns synchronization state.
/// HIPAR resourcesState/Returns resources state.
ViStatus _VI_FUNC rscmwam_QueryRecordingStatus(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32*	mainState,
	ViInt32*	syncState,
	ViInt32*	resourcesState
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    sprintf (command, "FETC:AUD:WAV%ld:REC:STAT:ALL?", instance);

    checkErr(rscmwam_QueryMeasStatus(instrSession, command, mainState, syncState,
        resourcesState));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/// HIFN  Queries the main measurement state and the measurement substates. Both
/// HIFN  measurement substates are relevant for running measurements only.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:AUDio:WAV:RECord:STATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR mainState/Returns main state.
ViStatus _VI_FUNC rscmwam_FetchRecordingStatus(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32*	mainState
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:AUD:WAV%ld:REC:STAT?", instance);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, mainState));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a source for the audio signal to be recorded.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:WAV:RECord:SOURce
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR source/Selects a source for the audio signal to be recorded.
/// HIPAR master/String parameter selecting the master application for the source
/// HIPAR master/SPEech.
ViStatus _VI_FUNC rscmwam_ConfigureRecordingSource(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	source,
	ViString	master
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSCMWAM_VAL_REC_SOURCE_ANALOG, RSCMWAM_VAL_REC_SOURCE_SPEECH),
			3, "Source");

	if(source == RSCMWAM_VAL_REC_SOURCE_SPEECH)
	{
		snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:WAV%ld:REC:SOUR %s,%s", instance, recSourceArr[source], master);
	}
	else
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:WAV%ld:REC:SOUR %s", instance, recSourceArr[source]);
	}

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the A/D conversion for the analog source of waveform
/// HIFN  recording.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_RECORDING_INPUT_LEVEL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:WAV:RECord:ILEVel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR inputLevel/Configures the A/D conversion for the analog source of waveform
/// HIPAR inputLevel/recording.
ViStatus _VI_FUNC rscmwam_ConfigureRecordingInputLevel(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64	inputLevel
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_RECORDING_INPUT_LEVEL, inputLevel),
    		3, "Input Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the cutoff frequency of the highpass filter for waveform
/// HIFN  recording.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_RECORDING_HIGHPASS_FILTER_FREQUENCY
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:WAV:RECord:FILTer:HPASs
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR frequency/Configures the cutoff frequency of the highpass filter for waveform
/// HIPAR frequency/recording.
ViStatus _VI_FUNC rscmwam_ConfigureRecordingHighpassFilterFrequency(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	frequency
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_RECORDING_HIGHPASS_FILTER_FREQUENCY, frequency),
    		3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the waveform file to be written.
/// HIFN     For convenience, use the alias @WAVEFORM for the path.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_RECORDING_FILE_PATH_NAME
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:WAV:RECord:FPAThname
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR filePathName/Selects the waveform file to be written.
ViStatus _VI_FUNC rscmwam_ConfigureRecordingFilePathName(
	ViSession	instrSession,
	ViInt32	instance,
	ViString	filePathName
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_RECORDING_FILE_PATH_NAME, filePathName),
    		3, "File Path Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts or stops the audio file playback.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_PLAYBACK_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SOURce:AUDio:WAV:PLAYback:STATe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Starts or stops the audio file playback.
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackEnabled(
	ViSession	instrSession,
	ViInt32	instance,
	ViBoolean	enabled
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViBoolean(instrSession, repCap, RSCMWAM_ATTR_PLAYBACK_ENABLED, enabled),
    		3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  File playback state.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_PLAYBACK_STATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SOURce:AUDio:WAV:PLAYback:STATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR state/Displays the file playback state.
ViStatus _VI_FUNC rscmwam_QueryPlaybackState(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32*	state
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_GetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_PLAYBACK_STATE, state),
    		3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a destination for the generated audio signal.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:WAV:PLAYback:DESTination
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR source/Selects a destination for the generated audio signal.
/// HIPAR master/String parameter selecting the master application for the source
/// HIPAR master/SPEech.
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackDestination(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	source,
	ViString	master
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSCMWAM_VAL_REC_SOURCE_ANALOG, RSCMWAM_VAL_REC_SOURCE_SPEECH),
			3, "Source");

	if (source == RSCMWAM_VAL_REC_SOURCE_SPEECH)
	{
		snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:WAV%ld:PLAY:DEST %s,%s", instance, recSourceArr[source], master);
	}
	else
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:AUD:WAV%ld:PLAY:DEST %s", instance, recSourceArr[source]);
	}

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the D/A conversion for the analog destination of waveform
/// HIFN  playback.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_PLAYBACK_OUTPUT_LEVEL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:WAV:PLAYback:OLEVel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR outputLevel/Configures the D/A conversion for the analog destination of waveform
/// HIPAR outputLevel/playback.
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackOutputLevel(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64	outputLevel
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViReal64(instrSession, repCap, RSCMWAM_ATTR_PLAYBACK_OUTPUT_LEVEL, outputLevel),
    		3, "Output Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the waveform file to be loaded.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_PLAYBACK_FILE_PATH_NAME
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:AUDio:WAV:PLAYback:FPAThname
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR filePathName/Selects the waveform file to be loaded.
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackFilePathName(
	ViSession	instrSession,
	ViInt32	instance,
	ViString	filePathName
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_SetAttributeViString(instrSession, repCap, RSCMWAM_ATTR_PLAYBACK_FILE_PATH_NAME, filePathName),
    		3, "File Path Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the progress of playing a waveform file.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWAM_ATTR_PLAYBACK_PROGRESS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:AUDio:WAV:PLAYback:PROGress:DREad?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR progress/Queries the progress of playing a waveform file.
ViStatus _VI_FUNC rscmwam_QueryPlaybackProgress(
	ViSession	instrSession,
	ViInt32     instance,
	ViInt32*	progress
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwam_GetAttributeViInt32(instrSession, repCap, RSCMWAM_ATTR_PLAYBACK_PROGRESS, progress),
    		3, "Progress");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViInt32 */
ViStatus _VI_FUNC rscmwam_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwam_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwam_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwam_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwam_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rscmwam_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwam_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwam_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwam_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, bufSize,
                                        value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwam_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwam_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
    		3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
    		4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}
/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rscmwam_GetAttributeRepCapName(ViSession instrSession,
    ViUInt32 attributeId,
    ViString repCapNameId,
    ViInt32 index,
    ViInt32 bufferSize,
    ViChar repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: rscmwam_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rscmwam_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwam_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwam_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwam_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwam_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwam_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    ViStatus error = VI_SUCCESS;
    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
        {
        checkErr(rscmwam_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
        }

    checkErr(rscmwam_DefaultInstrSetup(instrSession));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, driverRev), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrRev), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_SystemVersion (ViSession instrSession,
                                          ViInt32 length,
                                          ViChar systemVersion[])

{
    ViStatus    error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwam_GetAttributeViString(instrSession, "", RSCMWAM_ATTR_SYSTEM_VERSION,
    		length, systemVersion),
    		2, "System Version");
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errCode, 1024, errMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error List
 * Purpose:  This function reads all system messages and returns a list of
 *           comma separated strings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ErrorList (ViSession instrSession,
                                      ViInt32 bufferSize,
                                      ViChar errors[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 0, INT_MAX),
    		2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errors),
		3, "Null address for Errors");

    checkErr(rscmwam_GetAttributeViString( instrSession, "", RSCMWAM_ATTR_SYSTEM_ALL_ERRORS,
        bufferSize, errors));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    static RsCoreStringValueTable errorTable =
    {
        RSCMWAM_ERROR_CODES_AND_MSGS,
        {0, NULL}
    };

    (void)(RsCore_LockSession(instrSession));

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:
    (void)RsCore_UnlockSession(instrSession); // IGNORE: ERROR!!! Missing Lock
    return error;
}

/*****************************************************************************
 * Function: Reliability Indicator
 * Purpose:  This function converts a reliability code returned by an
 *           instrument driver function into a user-readable string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_ReliabilityIndicator (ViSession instrSession,
                                              ViInt32 reliabilityCode,
                                              ViChar reliabilityMessage[])
{
    ViStatus    error = VI_SUCCESS;
    ViInt16     cnt     = 0;
    ViBoolean   found = VI_FALSE;

    static      RsCoreStringValueTable reliabilityTable =
    {
        {0,       "OK"},
        {1,       "Measurement Timeout"},
        {2,       "Capture Buffer Overflow"},
        {3,       "Overdriven"},
        {4,       "Underdriven"},
        {6,       "Trigger Timeout"},
        {7,       "Acquisition Error"},
        {8,       "Sync Error"},
        {9,       "Uncal"},
        {15,       "Reference Frequency Error"},
        {16,       "RF Not Available"},
        {17,       "RF Level not Settled"},
        {18,       "RF Frequency not Settled"},
        {19,       "Call not Established"},
        {20,       "Call Type not Usable"},
        {21,       "Call Lost"},
        {23,       "Missing Option"},
        {24,       "Invalid RF Setting"},
        {26,       "Resource Conflict"},
        {27,       "No Sensor Connected"},
        {28,       "Unexpected Parameter Change"},
        {30,       "File not Found"},
        {31,       "No DTM reply"},
        {32,       "ACL Disconnected"},
        {40,       "ARB File CRC Error"},
        {42,       "ARB Header Tag Invalid"},
        {43,       "ARB Segment Overflow"},
        {44,       "ARB File not Found"},
        {45,       "ARB Memory Overflow"},
        {46,       "ARB Sample Rate out of Range"},
        {47,       "ARB Cycles out of Range"},
        {50,       "Startup Error"},
        {51,       "No Reply"},
        {52,       "Connection Error"},
        {53,       "Configuration Error"},
        {54,       "Filesystem Error"},
        {93,       "OCXO Oven Temperature too low"},
        {101,       "Firmware Error"},
        {102,       "Unidentified Error"},
        {103,       "Parameter Error"},
        {104,       "Not Functional"},
        {0, NULL}
    };

    (void) RsCore_LockSession(instrSession);

    for (cnt = 0; reliabilityTable[cnt].string; cnt++)
    {
        if (reliabilityTable[cnt].value == reliabilityCode)
        {
            sprintf (reliabilityMessage, "%s", reliabilityTable[cnt].string);
            if (reliabilityCode)
                error = RSCMWAM_ERROR_RELIABILITY_INDICATOR;
            found = VI_TRUE;
        }
    }

	if (!found)
	{
		sprintf(reliabilityMessage, "Unrecognized reliability indicator! Report problem to R&S support!");
		error = RS_WARN_UNKNOWN_INSTRUMENT_STATUS;
	}

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Preset
 * Purpose:  This function initiates an instrument reset.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_Preset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_SetAttributeViString(instrSession, "", RSCMWAM_ATTR_SYSTEM_PRESET, NULL));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwam_ClearStatus (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rscmwam_IDQueryResponse (ViSession instrSession,
                                             ViChar IDQueryResponse[])
{
    ViStatus    error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_QueryViString(instrSession, "*IDN?", 256, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwam_ProcessAllPreviousCommands (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rscmwam_QueryOPC (ViSession instrSession, ViInt32 *OPC)
{
    ViStatus    error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwam_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwam_SetVISATimeout (ViSession instrSession,
                                           ViUInt32 VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rscmwam_GetVISATimeout (ViSession instrSession,
                                           ViUInt32 *VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rscmwam_ConfigureErrorChecking (ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}
/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rscmwam_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rscmwam_ConfigureAutoSystemErrQuery(ViSession instrSession,
    ViBoolean autoSystErrQuery)
{
    ViStatus error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}
/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rscmwam_ConfigureMultiThreadLocking(ViSession instrSession, ViBoolean multiThreadLocking)
{
    ViStatus error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}



/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rscmwam_error_message, errorCode, bufferSize, description)
    );

Error:
    // Unlock Session
    // IGNORE: ERROR!!! Missing Lock
    return error;
}

ViStatus _VI_FUNC rscmwam_ClearError (ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN This function is used to read data from the instrument and write
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rscmwam_ReadToFileFromInstrument (ViSession instrSession,
                                                     ViString source,
                                                     ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    ViChar cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function is used to read data from the host computer and
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rscmwam_WriteFromFileToInstrument (ViSession instrSession,
                                                      ViString source,
                                                      ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    ViChar cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rscmwam_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

ViStatus _VI_FUNC rscmwam_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar* responseString = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ReadViStringUnknownLength(instrSession, &responseString));
    checkErr(RsCore_CopyToUserBufferAsciiData(rdBuf, numBytes, responseString));
    *bytesRead = (ViUInt32)strlen(responseString);

Error:
    if (responseString)
        free(responseString);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryViBoolean (ViSession instrSession,
                                           ViString command,
                                           ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryViInt32 (ViSession instrSession,
                                         ViString command,
                                         ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryViReal64 (ViSession instrSession,
                                          ViString command,
                                          ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryViString (ViSession instrSession,
                                          ViString command,
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar* pbuffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, command, &pbuffer));

    checkErr(rscmwam_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
    if (pbuffer)
        free(pbuffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rscmwam_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_close (ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

